/*
 *   Java App Blogging: Application for updating web pages via FTP. v0.8.0
 *   Copyright (C) 2002 Mark Andrew Wilson
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  ominous_squirrel@hotmail.com
 *  http://www.cs.indiana.edu/~marawils/jab/
 */

package mawilson.jab;

import java.io.*;
import java.util.*;
import java.text.DateFormat;
import java.text.ParseException;
import mawilson.file2array.*;

import com.enterprisedt.net.ftp.FTPException;
import com.enterprisedt.net.ftp.FTPClient;
import com.enterprisedt.net.ftp.FTPTransferType;
import com.enterprisedt.net.ftp.FTPConnectMode;

/**   This class contains methods for downloading, parsing, updating, then 
 *    uploading a web page with JAB.
 *    @author Mark Andrew Wilson
 **/

public class JABShared
{
    public static final String dayStart = new String("\n<!-- jab daily start mark: ");
    public static final String dayEnd = new String("\n<!-- jab end mark");
    public static final String donot = new String(" ; do not change this line -->\n");
    public static final String jabHere = new String("<!-- jab here -->");
    public static final String jabThere = new String("<!-- jab there -->");
    protected int port = 21;
    protected String FTPsite;
    protected String FTPfile;
    protected String FTPArchiveDir;
    
    protected File2Array blogFile;
    protected String template;
    protected static String dailyMarker;
    protected String username = "anonymous";
    protected FTPClient ftp;

    public JABShared(File2Array blogFile, String template, String FTPsite, String FTPfile, String FTPArchiveDir)
    {
	this.template = template;// 'eh.
	this.blogFile = blogFile;
	this.FTPsite = FTPsite;
	this.FTPfile = FTPfile;
	this.FTPArchiveDir = FTPArchiveDir;
	dailyMarker = new String();
    }
    //
    public JABShared(String FTPsite, String username, String password, String blogFilePath, String FTPArchiveDir, String templatePath, String filler) throws IOException, FTPException, ParseException
    {
	this.FTPsite = FTPsite;
	this.FTPfile = blogFilePath;
	this.FTPArchiveDir = FTPArchiveDir;
	dailyMarker = new String();

	ftp = new FTPClient(FTPsite, port);

	try
	    {
		ftp.login(username, password);
		ftp.setType(FTPTransferType.ASCII);
		ftp.setConnectMode(FTPConnectMode.PASV);
		this.blogFile=(File2Array) new FTP2Array(ftp, blogFilePath);
		this.template=new String(ftp.get(templatePath));
		blogFile = buildNewPage(blogFile, fillTemplate(filler, template), false);
		byte[] sendBuffer = (blogFile.toString()).getBytes();
		ftp.put(sendBuffer, blogFilePath);
	    }
	finally
	    {
		System.out.println("Closing FTP connection");
		ftp.quit();
	    }
    }

    public void setPort(int port)
    {
	this.port = port;
    }
        
    public String fillTemplate(String filler, String template) throws ParseException
    {
	//Add white space to make parser happy
	if(template.charAt(0) == '<' || template.charAt(0) == '$')
	    {
		template = new String(" " + template);
	    }

	StringTokenizer tokes = new StringTokenizer(template, "<");
	String returnString = new String();
	GregorianCalendar rightNow = new GregorianCalendar();

	//Date rightNow = new Date();
	

	if(filler.equals(returnString)) return returnString;
			
	while(tokes.hasMoreTokens())
	    {
		String currentToken = tokes.nextToken();
		
		if(currentToken.charAt(0) == '$')
		    {
			
			int split = currentToken.indexOf("$>");
			if(split != -1)
			    {
				String command = currentToken.substring(1, split);
				String leftover = new String();
				if(split+2<currentToken.length())
				    {leftover = currentToken.substring(split+2,currentToken.length());}
				
				String output = command2Action(command, filler, leftover, rightNow);
				
				returnString = returnString.concat(output);
			    }
			else
			    {throw(new ParseException("Invalid template: command not closed", tokes.countTokens()));}
		    }
		else
		    {
			String sooperString = new String("<");
			returnString = returnString.concat(sooperString);
			returnString = returnString.concat(currentToken);
		    }
	    }
	returnString = returnString.substring(1,returnString.length());
	return returnString;
    }

    public String command2Action(String command, String filler, String leftover, GregorianCalendar rightNow) throws ParseException
    {
	String output = new String();	
	
	Date dateNow = rightNow.getTime();
	String date = DateFormat.getDateInstance(DateFormat.FULL).format(dateNow);
	String time = DateFormat.getTimeInstance(DateFormat.SHORT).format(dateNow);
	dailyMarker = makeToday(rightNow); 
	    //DateFormat.getDateInstance(DateFormat.SHORT).format(rightNow);
	String unique = makeUnique(rightNow);

	if(command.equals("username")) output = username.concat(leftover);
	if(command.equals("date")) output= date.concat(leftover);
	if(command.equals("time")) output = time.concat(leftover);
	if(command.equals("body")) output = filler.concat(leftover);
	if(command.equals("unique")) output =unique.concat(leftover);
	
	if(command.equals("startdailyblock"))
	    {
		String start = new String(dayStart + dailyMarker + donot);
		output = start.concat(leftover);
	    }
	
	if(command.equals("enddailyblock"))
	    {
		String end = new String(dayEnd + donot);
		output = end.concat(leftover);
	    }
	
	if(output.equals(new String()))
	    {throw(new ParseException("Invalid command (" + command + ") in template.",0));}
	
	return output;
    }
    
    public String makeToday(GregorianCalendar rightNow)
    {
	String month = new Integer(rightNow.get(Calendar.MONTH)+1).toString();
	String day = new Integer(rightNow.get(Calendar.DAY_OF_MONTH)).toString();
	String year = new Integer(rightNow.get(Calendar.YEAR)).toString();

	return new String(month + "/" + day +"/"+year);
    }

    public String makeUnique(GregorianCalendar rightNow)
    {
	String wy = new Integer(rightNow.get(Calendar.WEEK_OF_YEAR)).toString();
	String dw = new Integer(rightNow.get(Calendar.DAY_OF_WEEK)).toString();
	String y = new Integer(rightNow.get(Calendar.YEAR)-2000).toString();
	String h = new Integer(rightNow.get(Calendar.HOUR_OF_DAY)).toString();
	String m = new Integer(rightNow.get(Calendar.MINUTE)).toString();
	String s = new Integer(rightNow.get(Calendar.SECOND)).toString();
	String mm = new Integer(rightNow.get(Calendar.MILLISECOND)).toString();

	return new String(wy + dw + y + h + m + s + mm);
    }

    public String makeArchiveName()
    {
	GregorianCalendar rightNow = new GregorianCalendar();
	
	String month = new Integer(rightNow.get(Calendar.MONTH)+1).toString();
	String day = new Integer(rightNow.get(Calendar.DAY_OF_MONTH)).toString();
	String year = new Integer(rightNow.get(Calendar.YEAR)).toString();

	String archiveName = new String(month + "_" + day + "_"+year + ".html");
	
	return new String(FTPArchiveDir + archiveName);
    }

    public void doSend(String username, String password, String filler) throws IOException, FTPException, ParseException
    {

	ftp = new FTPClient(FTPsite,port);
	
	try
	    {
		ftp.login(username, password);
		ftp.setType(FTPTransferType.ASCII);
		ftp.setConnectMode(FTPConnectMode.PASV);
		blogFile = buildNewPage(blogFile, fillTemplate(filler, template), false);
		byte[] sendBuffer = (blogFile.toString()).getBytes();
		ftp.put(sendBuffer, FTPfile);
	    }
	catch(IOException e)
	    {
		ftp.quit();
		throw e;
	    }
	catch(FTPException e)
	    {
		ftp.quit();
		throw e;
	    }
	catch(ParseException e)
	    {
		ftp.quit();
		throw e;
	    }
    }
    
    public void closeFTP() throws FTPException, IOException
    {
	ftp.quit();
    }

    public void doNewEntry(String filler) throws ParseException, IOException
    {
	blogFile = buildNewPage(blogFile, fillTemplate(filler, template), false);
    }
    
    public void doArchive(String username, String password, int cutLine) throws FTPException, IOException
    {
	ftp = new FTPClient(FTPsite,port);
	File2Array oldPage = new File2Array();
	File2Array newPage = new File2Array();
	String currentLine = new String();
	int n = 0;
	
	do
	    {
		currentLine = blogFile.getLine(n);
		oldPage.insertLine(currentLine, n);
		newPage.insertLine(currentLine, n);		
		n++;
	    }
	while(!currentLine.equals(jabHere) && n<blogFile.getLength());
	
	int oldPageIndex = n;
	
	while(n<cutLine)
	    {
		currentLine = blogFile.getLine(n);
		newPage.insertLine(currentLine, n);
		n++;
	    }
	
	int newPageIndex = n;
	
	while(!currentLine.equals(jabThere) && n<blogFile.getLength())
	    {
		currentLine = blogFile.getLine(n);
		oldPage.insertLine(currentLine, oldPageIndex);
		oldPageIndex++;
		n++;
	    }
	
	newPage.insertLine(jabThere, newPageIndex++);

	while(n<blogFile.getLength())
	    {
		currentLine = blogFile.getLine(n);
		oldPage.insertLine(currentLine, oldPageIndex);
		newPage.insertLine(currentLine, newPageIndex);		
		n++;
		oldPageIndex++;
		newPageIndex++;
	    }

	try
	    {		
		byte[] sendBuffer1 = newPage.toString().getBytes();
		byte[] sendBuffer2 = oldPage.toString().getBytes();
		
		ftp.login(username, password);
		ftp.setType(FTPTransferType.ASCII);
		ftp.setConnectMode(FTPConnectMode.PASV);
				
		ftp.put(sendBuffer1, FTPfile);
		System.out.println("erf: " + makeArchiveName());
		ftp.put(sendBuffer2, makeArchiveName());
		blogFile = newPage;
	    }
	catch(IOException e)
	    {
		ftp.quit();
		throw e;
	    }
	catch(FTPException e)
	    {
		ftp.quit();
		throw e;
	    }
    }

    public void renameTo(String sourcePath, String destPath) throws FTPException, IOException
    {
	String[] validCodes = {"350","250"};
	ftp.quote("rnfr " + sourcePath, validCodes);
	ftp.quote("rnto " + destPath, validCodes); 
    }
    
    public String getTemplate(String filler) throws ParseException
    {
	return fillTemplate(filler, template);
    }

    public Vector getBlog()
    {
	return blogFile.getVector();
    }

    public File2Array buildNewPage(File2Array blogFile, String insertMe, boolean neverRepress) throws IOException
    {	
	String startDay = new String(dayStart + dailyMarker + donot).trim();
	String endDay = new String(dayEnd + donot).trim();
	boolean repressLine = false;
	File2Array outputPage = new File2Array();
	int i = 0; // current line in blogFile
	int q = 0; // current line in outputPage

	if(insertMe.equals(new String())) neverRepress = true;

	while(i<blogFile.getLength())
	    {
		
		String blogLine = blogFile.getLine(i);
				
		if(blogLine.trim().equals(startDay)) repressLine = true;
		if(!repressLine || neverRepress) 
		    {
			outputPage.insertLine(blogLine, q);
			q++;
		    }
			if(blogLine.trim().equals(endDay)) repressLine = false;
		if(blogLine.trim().equals(jabHere))
		    {
			if(insertMe != null) 
			    {
				StringTokenizer tokes = new StringTokenizer(insertMe, new String("\n"));
				while(tokes.hasMoreTokens())
				    {
					outputPage.insertLine(tokes.nextToken(),q);
					q++;
				    }
			    }
		    }
		i++;
	    }
	return outputPage;
    }
}
