/*
 *   Java App Blogging: Application for updating web pages via FTP. v0.8.0
 *   Copyright (C) 2002 Mark Andrew Wilson
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  ominous_squirrel@hotmail.com
 *  http://www.cs.indiana.edu/~marawils/jab/
 */

package mawilson.jab;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.text.ParseException;
import mawilson.jab.JABShared;
import mawilson.file2array.File2Array;
import com.enterprisedt.net.ftp.FTPException;
import FlowOrFillLayout;

import java.net.URL;

/**  Swing-based GUI for the Java App Blogging web page updating
 *   package. 
 *   @author Mark Andrew Wilson
 *   
 **/
public class JABSwing implements ActionListener, MouseListener
{
    protected JTextField usernameInput;
    protected JPasswordField passwordInput;
    // Settings window text fields. 
    protected JTextField sTextField1,sTextField2,sTextField3,sTextField4;
    protected JTextField sTextField5,sTextField6,sTextField7, sTextField8;
    protected JTextArea entryInput;
    protected JTextArea errorTextArea;
    protected JButton sendButton;
    protected JMenuItem archiveMenuItem;
    protected JList archiveList;
    protected JPopupMenu popup;

    protected JFrame archiveFrame;
    protected JFrame viewFrame;
    protected JFrame settingsWindow;
    protected JEditorPane viewHTMLPane;
    protected JButton viewButton;
    protected JButton newEntryButton;
    //The JABShared object contains a mishmash of
    //methods for uploading and downloading the pages
    //and parsing the template.
    protected static JABShared jabshared;
    
    //Have we downloaded the page yet?
    protected boolean pageLoaded = false;
    protected String errorString = " ";
    protected String WWWURL;
    protected String templateLocation = "sampleTemplate.html";
    protected String FTPsite;
    protected String FTPport = "21";
    protected String FTPfile;
    protected String FTPtempfile;
    protected String FTPArchiveDir;
    protected String settingsFile = "JABSwing.dat";
    
    //Are we only going to show the setup window?
    protected boolean setupOnly = false;
    
    public static void main(String[] args)
    {	
	String WURL;
	String templateURL;
	String FTPURL;
	String FTPLocation;
	JABSwing app;
	boolean setupOnly = false;
	
	switch(args.length)
	    {
		//Open setup window only
	    case 0:
		setupOnly = true;
		app = new JABSwing();
		break;
		//Load initialization file
	    case 1:
		app = new JABSwing(args[0]);
		break;
		//Initialize JABSwing using command line parameters
	    case 4:
		//WURL = args[0];
		//templateURL = args[1];
		//FTPURL = args[2];
		//FTPLocation = args[3];
		//app = new JABSwing(WURL, templateURL, FTPURL, FTPLocation, FTPArchiveDir);
		//break;
		//Bad command line parameters. Display usage info and exit.
	    default:
		usage();
		return;
	    }
	
	try
	    {
		UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
	    }
	catch(java.lang.ClassNotFoundException e)
	    {app.onError(e);}
	catch(java.lang.InstantiationException e)
	    {app.onError(e);}
	catch(java.lang.IllegalAccessException e)
	    {app.onError(e);}
	catch(javax.swing.UnsupportedLookAndFeelException e)
	    {app.onError(e);}
	
	JFrame frame = new JFrame("JABSwing");
	JPanel innerPanel = new JPanel();
	innerPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
	app.createComponents(innerPanel);
	frame.getContentPane().add(innerPanel);
	frame.setJMenuBar(app.createMenuBar());
		
	frame.addWindowListener(new WindowAdapter() 
	    {
		public void windowClosing(WindowEvent e) 
		{
		    System.exit(0);
		}
	    });
	
	frame.pack();
	
	if(!setupOnly)
	    {
		frame.setVisible(true);
	    }
    }
    
    /**   Constructor that only shows the settings window. 
     **/
    public JABSwing()
    {
	setupOnly = true;
	buildSettingsWindow();
    }

    /**   Constructor that takes the URL of the page and template, and the FTP
     *    address and path to send the new page to. 
    **/
    public JABSwing(String WWWURL, String templateLocation, String FTPsite, String FTPtempfile, String FTPArchiveDir)
    {
	this.WWWURL = WWWURL;
	this.templateLocation = templateLocation;
	this.FTPsite = FTPsite;
	this.FTPfile = FTPfile;
	this.FTPArchiveDir = FTPArchiveDir;
    }
   
    /**  Constructor that loads settings using initialization file.
     **/
    public JABSwing(String settingsFile)
    {
	this.settingsFile = settingsFile;
		
	try
	    {
		File2Array settings = new File2Array(settingsFile);
		this.WWWURL = settings.getLine(0);
		this.templateLocation = settings.getLine(1);
		this.FTPsite = settings.getLine(2);
		this.FTPfile = settings.getLine(3);
		this.FTPport = settings.getLine(4);
		this.FTPtempfile = settings.getLine(5);
		this.FTPArchiveDir = settings.getLine(6);
	    }
	catch(IOException e)
	    {
		infoAppend("JABSwing can't find initialization file.");
		onError(e);
	    }
	catch(ArrayIndexOutOfBoundsException e)
	    {
		infoAppend("JABSwing initialization file invalid.");
		onError(e);
	    }
    }
    
    /** Builds the GUI components for the main window and
     *  builds the viewing window.
     **/
    public void createComponents(Container cp)
    {
	JLabel label1 = new JLabel("Username:");
	JLabel label2 = new JLabel("Password:");
	
	usernameInput = new JTextField(10);
	passwordInput = new JPasswordField(10);
	
	//JButton enterButton = new JButton("Enter");
	//enterButton .addActionListener(this);
	//enterButton.setEnabled(true);
	
	sendButton = new JButton("Send");
	sendButton.addActionListener(this);
	sendButton.setEnabled(false);

	viewButton = new JButton("View");
	viewButton.addActionListener(this);
	viewButton.setEnabled(false);

	newEntryButton = new JButton("New Entry");
	newEntryButton.addActionListener(this);
	newEntryButton.setEnabled(false);

	entryInput = new JTextArea();
	entryInput.addMouseListener(this);
	entryInput.setLineWrap(true);
	entryInput.setWrapStyleWord(true);
	JScrollPane entryScroller = new JScrollPane(entryInput);
	entryScroller.setVerticalScrollBarPolicy(
			JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        entryScroller.setPreferredSize(new Dimension(250, 250));

	popup = new JPopupMenu();
	popup = (JPopupMenu) fillEditMenu(popup);
	
	FlowOrFillLayout layout = new FlowOrFillLayout();
	cp.setLayout(layout);
	
	cp.add(label1);
        cp.add(usernameInput);
	cp.add(label2);
	cp.add(passwordInput);
	layout.addLayoutComponent(passwordInput, layout.CENTER_LINE);
		
	cp.add(entryScroller);
	layout.addLayoutComponent(entryScroller, layout.FILL_LINE);
	
	cp.add(sendButton);
	cp.add(viewButton);
	cp.add(newEntryButton);
	layout.addLayoutComponent(newEntryButton, layout.CENTER_LINE);
	
	createViewWindow();
    }


    protected void buildSettingsWindow()
    {
	if(settingsWindow == null)
	    {
		settingsWindow = new JFrame("JABSwing settings");
		Container cp = settingsWindow.getContentPane();

		FlowOrFillLayout layout = new FlowOrFillLayout(5, 2);
		cp.setLayout(layout);
					
		JLabel label1 = new JLabel("WWW address of page: ");
		layout.addLayoutComponent(label1, layout.NORMAL);
		cp.add(label1);
		sTextField1 = new JTextField(WWWURL, 20);
		layout.addLayoutComponent(sTextField1, layout.FILL_LINE);
		cp.add(sTextField1);
		JLabel label2 = new JLabel("HTTP address of page to be loaded and altered by JABSwing");
		layout.addLayoutComponent(label2, layout.LINE_BREAK);
		cp.add(label2);
		
		JLabel blanklabel1 = new JLabel(" ");
		layout.addLayoutComponent(blanklabel1, layout.LINE_BREAK);
		cp.add(blanklabel1);

		JLabel label3 = new JLabel("Template file location: ");
		layout.addLayoutComponent(label3, layout.NORMAL);
		cp.add(label3);
		sTextField2 = new JTextField(templateLocation, 20);
		layout.addLayoutComponent(sTextField2, layout.FILL_LINE);
		cp.add(sTextField2);
		JLabel label4 = new JLabel("Local filename or HTTP address of JABSwing template file");
		layout.addLayoutComponent(label4, layout.LINE_BREAK);
		cp.add(label4);
		
		JLabel blanklabel2 = new JLabel(" ");
		layout.addLayoutComponent(blanklabel2, layout.LINE_BREAK);
		cp.add(blanklabel2);

		JLabel label5 = new JLabel("Address of FTP server: ");
		layout.addLayoutComponent(label5, layout.NORMAL);
		cp.add(label5);
		sTextField3 = new JTextField(FTPsite, 20);
		layout.addLayoutComponent(sTextField3, layout.NORMAL);
		cp.add(sTextField3);
		JLabel label6 = new JLabel(" Port: ");
		layout.addLayoutComponent(label6, layout.NORMAL);
		cp.add(label6);
		sTextField6 = new JTextField(FTPport, 4);
		layout.addLayoutComponent(sTextField6, layout.FILL_LINE);
		cp.add(sTextField6);

		JLabel label7 = new JLabel("e.g. ftp.yourserver.edu");
		layout.addLayoutComponent(label7, layout.LINE_BREAK);
		cp.add(label7);
		
		JLabel blanklabel3 = new JLabel(" ");
		layout.addLayoutComponent(blanklabel3, layout.LINE_BREAK);
		cp.add(blanklabel3);
		
		JLabel label8 = new JLabel("Full path on FTP server: ");
		layout.addLayoutComponent(label8, layout.NORMAL);
		cp.add(label8);
		sTextField4 = new JTextField(FTPfile, 20);
		layout.addLayoutComponent(sTextField4, layout.FILL_LINE);
		cp.add(sTextField4);
		
		JLabel label9 = new JLabel("Full path of page to be loaded and altered by JABSwing on FTP server"); 
		layout.addLayoutComponent(label9, layout.FILL_LINE);
		cp.add(label9);
		JLabel label10 = new JLabel("e.g. /home/user/youraccount/www/index.html"); 
		layout.addLayoutComponent(label10, layout.FILL_LINE);
		cp.add(label10);

		JLabel blanklabel4 = new JLabel(" ");
		layout.addLayoutComponent(blanklabel4, layout.LINE_BREAK);
		cp.add(blanklabel4);

		JLabel label11 = new JLabel("Full temporary file path on FTP server: ");
		layout.addLayoutComponent(label11, layout.NORMAL);
		cp.add(label11);
		sTextField7 = new JTextField(FTPtempfile, 20);
		layout.addLayoutComponent(sTextField7, layout.FILL_LINE);
		cp.add(sTextField7);
		
		JLabel label12 = new JLabel("Full path on FTP server that JAB will use as a temporary file during upload"); 
		layout.addLayoutComponent(label12, layout.FILL_LINE);
		cp.add(label12);
		JLabel label13 = new JLabel("e.g. /home/user/youraccount/www/testindex.html"); 
		layout.addLayoutComponent(label13, layout.FILL_LINE);
		cp.add(label13);

		JLabel blanklabel5 = new JLabel(" ");
		layout.addLayoutComponent(blanklabel5, layout.LINE_BREAK);
		cp.add(blanklabel5);
		
		JLabel label16 = new JLabel("Archive directory on FTP server: ");
		layout.addLayoutComponent(label16, layout.NORMAL);
		cp.add(label16);
		sTextField8 = new JTextField(FTPArchiveDir, 20);
		layout.addLayoutComponent(sTextField8, layout.FILL_LINE);
		cp.add(sTextField8);
		
		JLabel label17 = new JLabel("Full path to directory on FTP server for JAB to place archive files"); 
		layout.addLayoutComponent(label17, layout.FILL_LINE);
		cp.add(label17);
		JLabel label18 = new JLabel("e.g. /home/user/youraccount/www/blogarchive/"); 
		layout.addLayoutComponent(label18, layout.FILL_LINE);
		cp.add(label18);

		JLabel blanklabel6 = new JLabel(" ");
		layout.addLayoutComponent(blanklabel6, layout.LINE_BREAK);
		cp.add(blanklabel6);

		JLabel label14 = new JLabel(" Save settings as: ");
		layout.addLayoutComponent(label14, layout.NORMAL);
		cp.add(label14);		    
		sTextField5 = new JTextField(settingsFile, 20);
		layout.addLayoutComponent(sTextField5, layout.FILL_LINE);
		cp.add(sTextField5);
		
		JLabel label15 = new JLabel("File will be overwritten with new settings");
		layout.addLayoutComponent(label15, layout.LINE_BREAK);
		cp.add(label15);

		JLabel blanklabel7 = new JLabel(" ");
		layout.addLayoutComponent(blanklabel7, layout.LINE_BREAK);
		cp.add(blanklabel7);

		JButton saveButton = new JButton("Save settings");
		saveButton.addActionListener(this);
		layout.addLayoutComponent(saveButton, layout.NORMAL);
		cp.add(saveButton);
		JButton cancelButton = new JButton("Cancel");
		cancelButton.addActionListener(this);
		layout.addLayoutComponent(cancelButton, layout.CENTER_LINE);
		cp.add(cancelButton);
	
		if(setupOnly)
		    {
			settingsWindow.addWindowListener(new WindowAdapter() 
			    {
				public void windowClosing(WindowEvent e) 
				{
				    System.exit(0);
				}
			    });
		    }
		else
		    {
			settingsWindow.addWindowListener(new WindowAdapter() 
			    {
				public void windowClosing(WindowEvent e) 
				{
				    settingsWindow.setVisible(false);
				}
			    });
			
		    }
		settingsWindow.pack();	
	    }
	settingsWindow.setVisible(true);
	
    }
    
    
    /**  Builds the archive window.
     **/
    protected void createArchiveWindow()
    {
	if (archiveFrame == null)
	    {
		archiveFrame = new JFrame("JABSwing Archive");
		Container cp = archiveFrame.getContentPane();
		
		FlowOrFillLayout layout = new FlowOrFillLayout();
		cp.setLayout(layout);
		
		archiveList = new JList(jabshared.getBlog());
		
		JScrollPane archiveScroller = new JScrollPane(archiveList);
		archiveScroller.setVerticalScrollBarPolicy(
							   JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		archiveScroller.setPreferredSize(new Dimension(640, 320));
		
		JLabel label = new JLabel("Select a line. Everything below that line will be archived.");
		layout.addLayoutComponent(label, layout.CENTER_LINE);
		cp.add(label);
		
		layout.addLayoutComponent(archiveScroller, layout.FILL_LINE);
		cp.add(archiveScroller);
		
		JButton okButton = new JButton("Okay");
		okButton.addActionListener(this);
		layout.addLayoutComponent(okButton, layout.NORMAL);
		cp.add(okButton);
		
		JButton neverButton = new JButton("Never mind");
		neverButton.addActionListener(this);
		layout.addLayoutComponent(neverButton, layout.CENTER_LINE);
		cp.add(neverButton);
		
		archiveFrame.addWindowListener(new WindowAdapter() 
		    {
			public void windowClosing(WindowEvent e) 
			{
			    archiveFrame.setVisible(false);
			}
		    });
		archiveFrame.pack();
	    }
	archiveFrame.setVisible(true);
    }

    
    /**  Called by createComponents to build viewing window.
     **/
    protected void createViewWindow()
    {
	viewFrame = new JFrame();
	viewHTMLPane = new JEditorPane();
	viewHTMLPane.setEditable(false);
	viewHTMLPane.setContentType("text/html");
	JScrollPane HTMLScroller = new JScrollPane(viewHTMLPane);
	HTMLScroller.setPreferredSize(new Dimension(650, 240));
	viewFrame.getContentPane().add(HTMLScroller);
	
	viewFrame.addWindowListener(new WindowAdapter() 
	    {
		public void windowClosing(WindowEvent e) 
		{
		    viewFrame.setVisible(false);
		}
	    });
	viewFrame.pack();
    }
    
    /**   Called by createComponents to build main window's menubar
     **/
    protected JMenuBar createMenuBar()
    {
	JMenuBar menuBar = new JMenuBar();
	
	JMenu fileMenu = new JMenu("File");
	fileMenu.setMnemonic(KeyEvent.VK_F);
	menuBar.add(fileMenu);
	JMenuItem loadPageItem = new JMenuItem("Load page", KeyEvent.VK_L);
	loadPageItem.addActionListener(this);
	fileMenu.add(loadPageItem);
	archiveMenuItem = new JMenuItem("Archive", KeyEvent.VK_A);
	archiveMenuItem.addActionListener(this);
	archiveMenuItem.setEnabled(false);
	fileMenu.add(archiveMenuItem);
	JMenuItem setupItem = new JMenuItem("Setup", KeyEvent.VK_U);
	setupItem.addActionListener(this);
	fileMenu.add(setupItem);
	JMenuItem quitItem = new JMenuItem("Quit", KeyEvent.VK_Q);
	quitItem.addActionListener(this);
	fileMenu.add(quitItem);
	
	JMenu editMenu = new JMenu("Edit");
	editMenu.setMnemonic(KeyEvent.VK_E);
	menuBar.add((JMenu) fillEditMenu(editMenu));
		
	JMenu helpMenu = new JMenu("Help");
	helpMenu.setMnemonic(KeyEvent.VK_H);
	menuBar.add(helpMenu);
	JMenuItem aboutItem = new JMenuItem("About", KeyEvent.VK_A);
	aboutItem.addActionListener(this);
	helpMenu.add(aboutItem);
	JMenuItem gplItem = new JMenuItem("License");
	gplItem.addActionListener(this);
	helpMenu.add(gplItem);
	JMenuItem helpItem = new JMenuItem("View help", KeyEvent.VK_H);
	helpItem.addActionListener(this);
	helpMenu.add(helpItem);

	return menuBar;
    }

    /**  Fills a JMenu or a JPopupMenu with the edit menu and 
     *   returns it.
    **/
    protected JComponent fillEditMenu(JComponent editMenu)
    {
	JMenuItem cutItem = new JMenuItem("Cut", KeyEvent.VK_T);
	cutItem.addActionListener(this);
	editMenu.add(cutItem);
	JMenuItem copyItem = new JMenuItem("Copy", KeyEvent.VK_C);
	copyItem.addActionListener(this);
	editMenu.add(copyItem);
	JMenuItem pasteItem = new JMenuItem("Paste", KeyEvent.VK_P);
	pasteItem.addActionListener(this);
	editMenu.add(pasteItem);
	JMenuItem pasteAsItem = new JMenuItem("Paste as link", KeyEvent.VK_L);
	pasteAsItem.addActionListener(this);
	editMenu.add(pasteAsItem);

	return editMenu;
    }

    /** **/
    protected void askAboutRename()
    {
	JOptionPane op = new JOptionPane();
	
	int n = op.showConfirmDialog(sendButton, "Overwrite permanent file with uploaded file?", "Upload complete", op.YES_NO_OPTION);
	
	try
	    {
		switch(n)
		    {
		    case JOptionPane.YES_OPTION:
			jabshared.renameTo(FTPtempfile, FTPfile);
			//jabshared.closeFTP();
			break;
		    default:
		    }
	    }
	catch(FTPException e) {onError(e);}
	catch(IOException e) {onError(e);}
	finally
	    {
		try {jabshared.closeFTP();}
		catch(FTPException e) {onError(e);}
		catch(IOException e) {onError(e);}
	    }
    }

    public void actionPerformed(ActionEvent e)
    {
	String actionCommand = e.getActionCommand();
	String username = "";
	String password = "";
	String filler = "";
	//System.out.println(actionCommand);

	if(actionCommand.equals("Cancel"))
	    {
		if(setupOnly) System.exit(0);
		else settingsWindow.setVisible(false);
		enableButtons();
	    }

	if(actionCommand.equals("Save settings"))
	    {
		this.WWWURL = sTextField1.getText();
		this.templateLocation = sTextField2.getText();
		this.FTPsite = sTextField3.getText();
		this.FTPfile = sTextField4.getText();
		this.settingsFile = sTextField5.getText();
		this.FTPport = sTextField6.getText();
		this.FTPtempfile = sTextField7.getText();
		this.FTPArchiveDir = sTextField8.getText();

		try
		    {
			BufferedWriter out = new BufferedWriter(new FileWriter(settingsFile));
			out.write(WWWURL + "\n");
			out.write(templateLocation + "\n");
			out.write(FTPsite + "\n");
			out.write(FTPfile + "\n");
			out.write(FTPport + "\n");
			out.write(FTPtempfile + "\n");
			out.write(FTPArchiveDir + "\n");
			out.close();
		    }
		catch(IOException ex)
		    {
			onError(ex);
		    }

		if(setupOnly) System.exit(0);
		else settingsWindow.setVisible(false);
		    
		enableButtons();
	    }

	if(actionCommand.equals("Quit"))
	    {
		System.exit(0);
	    }

	if(actionCommand.equals("Never mind"))
	    {
		archiveFrame.setVisible(false);
		enableButtons();
	    }
	
	if(!setupOnly)
	    {
		username = new String(usernameInput.getText());
		//jblogshared.setUsername(username);
		
		password = new String(passwordInput.getPassword());
		
		filler = entryInput.getText();

		sendButton.setEnabled(false);
		viewButton.setEnabled(false);
		newEntryButton.setEnabled(false);
		archiveMenuItem.setEnabled(false);

	    }

	if(actionCommand.equals("Cut"))
	    {
		entryInput.cut();
		enableButtons();
	    }

	if(actionCommand.equals("Copy"))
	    {
		entryInput.copy();
		enableButtons();
	    }

	if(actionCommand.equals("Paste"))
	    {
		entryInput.paste();
		enableButtons();
	    }
	
	if(actionCommand.equals("Paste as link"))
	    {
		String insideText = entryInput.getSelectedText();
		
		if(insideText == null)
		    {
			entryInput.insert("<a href="+ (char) 34,entryInput.getCaretPosition());
			entryInput.paste();
			entryInput.insert((char) 34 + ">", entryInput.getCaretPosition());
			entryInput.paste();
			entryInput.insert("</a>", entryInput.getCaretPosition());
		    }
		else
		    {
			entryInput.replaceSelection("<a href="+ (char) 34);
			entryInput.paste();
			entryInput.insert((char) 34 + ">", entryInput.getCaretPosition());
			entryInput.insert(insideText, entryInput.getCaretPosition());
			entryInput.insert("</a>", entryInput.getCaretPosition());
		    }
		
		enableButtons();
	    }
	
	if(actionCommand.equals("Load page"))
	    {
		
		File2Array blogFile = null;
		String template = null;

		try
		    {
			blogFile = new File2Array(WWWURL);
			template = new File2Array(templateLocation).toString();
			pageLoaded = true;
		    }
		catch(IOException ex)
		    {
			infoAppend("Can't find file.");
			onError(ex);
		    }
		
		this.jabshared = new JABShared(blogFile, template, FTPsite, FTPtempfile, FTPArchiveDir);
		jabshared.setPort(new Integer(FTPport).intValue());
		enableButtons();
	    }

	if(actionCommand.equals("Setup"))
	    {
		buildSettingsWindow();
		enableButtons();
	    }

	if(actionCommand.equals("Archive")&& !username.equals(new String()) && !password.equals(new String()))
	    {
		createArchiveWindow();
		enableButtons();
	    }

	if(actionCommand.equals("Okay"))
	    {
		try
		    {
			int cutLine = archiveList.getSelectedIndex();
			jabshared.doArchive(username, password, cutLine);
			archiveFrame.setVisible(false);
			askAboutRename();
		    }
		catch(FTPException ex)
		    {onError(ex);}
		catch(IOException ex)
		    {onError(ex);}
		enableButtons();
	    }
	
	if(actionCommand.equals("Send") && !username.equals(new String()) && !password.equals(new String()))
	    {
		try
		    {
			jabshared.doSend(username, password, filler);
			askAboutRename();
			entryInput.setText(null);
		    }
		catch(IOException ex)
		    {onError(ex);}
		catch(java.text.ParseException ex)
		    {onError(ex);}
		catch(FTPException ex)
		    {onError(ex);}
		
		enableButtons();
	    }
	
	if(actionCommand.equals("View"))
	    {
		viewFrame.dispose();
		createViewWindow();
		viewFrame.setTitle("Viewing entry...");
		
		try
		    {
			viewHTMLPane.setContentType("text/html");
			viewHTMLPane.setText(jabshared.getTemplate(filler));
		    }
		catch(Exception ex)
		    {onError(ex);}

		viewFrame.setVisible(true);
		enableButtons();
	    }
	
	if(actionCommand.equals("About"))
	    {
		viewFrame.dispose();
		createViewWindow();
		viewFrame.setTitle("About JAB");
		viewHTMLPane.setText(about());
		viewFrame.setVisible(true);
		enableButtons();
	    }
	
	if(actionCommand.equals("View help"))
	    {
		viewFrame.dispose();
		createViewWindow();
		viewFrame.setTitle("JABSwing help");
		try
		    {
			URL url = new URL("file:help.html");
			viewHTMLPane.setPage(url);
		    }    
		catch(IOException ex)
		    {
			infoAppend("Help file missing.");
		    	onError(ex);
		    }

		viewFrame.setVisible(true);
		enableButtons();
	    }

	if(actionCommand.equals("License"))
	    {
		viewFrame.dispose();
		createViewWindow();
		viewFrame.setTitle("GNU GENERAL PUBLIC LICENSE");
		try
		    {
			URL url = new URL("file:license.txt");
			viewHTMLPane.setPage(url);
		    }    
		catch(IOException ex)
		    {
			infoAppend("license.txt is missing.");
		    	onError(ex);
		    }

		viewFrame.setVisible(true);
		enableButtons();
	    }
	
	if(actionCommand.equals("New Entry"))
	    {
		try
		    {
			jabshared.doNewEntry(filler);
			entryInput.setText(null);
		    }
		catch(ParseException ex)
		    {onError(ex);}
		catch(IOException ex)
		    {onError(ex);}
				
		enableButtons();
	    }
    }

    /**   Popup menu trigger check.
     **/
    public void mousePressed(MouseEvent e)
    {
        maybeShowPopup(e);
    }

    /**   Popup menu trigger check.
     **/
    public void mouseReleased(MouseEvent e)
    {
        maybeShowPopup(e);
    }

    public void mouseClicked(MouseEvent e)
    {}

    public void mouseEntered(MouseEvent e)
    {}

    public void mouseExited(MouseEvent e)
    {}
        
    protected void maybeShowPopup(MouseEvent e)
    {
        if (e.isPopupTrigger())
	    {
		popup.show(e.getComponent(),
			   e.getX(), e.getY());
	    }
    }

    /**   Checks to see if the page is loaded and if so,
     *    method enables the relevant buttons.
     **/
    protected void enableButtons()
    {
	if(pageLoaded)
	    {
		sendButton.setEnabled(true);
		archiveMenuItem.setEnabled(true);
		viewButton.setEnabled(true);
		newEntryButton.setEnabled(true);
	    }
    }

    /**  Something bad happened. Open the error window.
     **/
    public void onError(Exception e)
    {
	JOptionPane op = new JOptionPane();
	String errorText = errorString + e.toString();

	//int n = 
	op.showMessageDialog(sendButton, errorText, "JABSwing Error", op.ERROR_MESSAGE);
	
    }

    /**  Add some random information to the error window.
     **/
    public void infoAppend(String string)
    {
	errorString = string + "\n";
    }
    
    /**  Make a Gridbag label.
     **/
    public JLabel gridLabel(String string, GridBagLayout gridbag, GridBagConstraints c)
    {
	//String string = new String(chars);
	JLabel label = new JLabel(string);
	gridbag.setConstraints(label, c);
	return label;
    }

    public JButton gridButton(String string, GridBagLayout gridbag, GridBagConstraints c)
    {
	JButton button = new JButton(string);
	button.addActionListener(this);
	gridbag.setConstraints(button, c);
	return button;
    }
    
    /**   Sends command line usage information to System.out
     **/
    public static void usage()
    {
	System.out.println("Usage: ");
	System.out.println("");
	System.out.println("JABSwing <Settings file>");
	System.out.println("");
	System.out.println("JABSwing <blog WWW address> <template file location> <FTP server address> <blog file location on server>");
    }

    public String about()
    {
	return 
	    new String("<html><body><center>Java App Blogging<br>"
		       + "Version 0.8.0<br>"
		       + "Copyright (C) 2002 Mark Andrew Wilson<br></center>"
		       + "<p>JAB comes with ABSOLUTELY NO WARRANTY. "
		       + "This is free software, and you are welcome to "
		       + "redistribute it under certain conditions. "
		       + "Access the 'License' menu option for details.</p>"
		       + "<center><br>ominous_squirrel@hotmail.com<br>"
		       + "http://www.cs.indiana.edu/~marawils/jab/<br>"
		       + "</center></body></html>");
    }
        
}
